﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.07.13.
 * Idő: 14:58
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;
//
using TetheredSun;
using TetheredSun.Mathematics;

namespace TetheredSun.Controls
{
	/// <summary>
	/// Description of DataPointList.
	/// </summary>
	public class DataPointList : List<Point<double>>, IDataPointList
	{
		protected object syncRoot = new Object();
		protected bool assumeXMonotonicity = true;
		
		
		public DataPointList(int capacity) : base(capacity) { }
		
		public DataPointList() : base() { }
		
		public bool AssumeXMonotonicity {
			get { return assumeXMonotonicity; }
			set { assumeXMonotonicity = value; }
		}
			
		public object SyncRoot {
			get { return syncRoot; }
		}
		
		public void Add(double x, double y)
		{
			this.Add(new Point<double>(x, y));
		}
		
		public List<Point<double>> GetPointsInRange(double xMinimum, double xMaximum)
		{
			double x;
			List<Point<double>> pointsInRange = new List<Point<double>>();
			bool first = true;
			
			for (int i = 0; i < this.Count; i++) {
				x = this[i].X;
				if (x >= xMinimum) {
					if (x <= xMaximum) {
						if (first) {
							first = false;
							if (i > 0) pointsInRange.Add(this[i - 1]);
						}
						pointsInRange.Add(this[i]);
					} else if (assumeXMonotonicity) {
						if (i < this.Count - 1) pointsInRange.Add(this[i + 1]);
						return pointsInRange;
					}
				}
			}
			
			return pointsInRange;
		}
		
		public void SetItems(double[] x, double[] y)
		{
			int length = Math.Min(x.Length, y.Length);
			
			this.Clear();
			
			for (int i = 0; i < length; i++) {
				this.Add(new Point<double>(x[i], y[i]));
			}
		}
		
		public void SetItems(double[] y, double x0, double dx)
		{
			this.Clear();
			
			for (int i = 0; i < y.Length; i++) {
				this.Add(new Point<double>(x0 + i * dx, y[i]));
			}
		}
		
		public void SetItems(double[] y, double x0, double dx, int maxPoints)
		{
			if (y.Length <= maxPoints) {
				SetItems(y, x0, dx);
			} else {
				int decimation = (int)Math.Ceiling(((double)y.Length) / maxPoints);
				int length = y.Length / decimation;

				SetItems(y.Decimate(decimation), x0, dx * decimation);
			}
		}
		
		public bool UpdateBounds(out double xMinimum, out double xMaximum, out double yMinimum, out double yMaximum)
		{
			if (this.Count < 1) {
				xMinimum = Double.NaN;
				xMaximum = Double.NaN;
				yMinimum = Double.NaN;
				yMaximum = Double.NaN;
				return false;
			}

			xMinimum = this[0].X;
			xMaximum = xMinimum;
			yMinimum = this[0].Y;
			yMaximum = yMinimum;

			for (int i = 0; i < this.Count; i++) {
				if (this[i].X < xMinimum) {
					xMinimum = this[i].X;
				} else if (this[i].X > xMaximum) {
					xMaximum = this[i].X;
				}

				if (this[i].Y < yMinimum) {
					yMinimum = this[i].Y;
				} else if (this[i].Y > yMaximum) {
					yMaximum = this[i].Y;
				}
			}

			return true;
		}
		
	}
}
