﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.10.16.
 * Idő: 11:31
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.IO
{
	
	public class IniParameter
	{
		public static char EscapeCharacter = '\\';
		public static string CommentSymbol = ";";
		public static string NewLineSymbol = "¶";
		public static string QuoteSymbol = "\"";
		public static string Separator = "=";
		//
		protected string key;
		protected string value;
		protected bool isQuoted = false;
		
		public IniParameter(string key, string value, bool isQuoted)
		{
			this.key = key;
			this.value = value;
			this.isQuoted = isQuoted;
		}
		
		public IniParameter(string key, string value) : this(key, value, false) { }
		
		
		public string Key {
			get { return key; }
		}
		
		public bool IsQuoted {
			get { return isQuoted; }
			set { isQuoted = value; }
		}
		
		
		public string Value {
			get { return value; }
		}
		
		public static string EscapeNewLine(string text)
		{
			return text.Replace(Environment.NewLine, NewLineSymbol);
		}
		
		public static string UnescapeNewLine(string text)
		{
			return text.Replace(NewLineSymbol, Environment.NewLine);
		}
		
		public string ToString(bool quoteValue)
		{
			string valueString = EscapeNewLine(value);
			
			if (quoteValue) valueString = String.Format("\"{0}\"", valueString);
			
			return String.Format("{0} {1} {2}", key, Separator, valueString);
		}
		
		public override string ToString()
		{
			return ToString(isQuoted);
		}
		
		
		public static bool TryParse(string text, out IniParameter parameter)
		{
			if (String.IsNullOrEmpty(text)) {
				parameter = null;
				return false;
			}
			
			if (text.StartsWith(CommentSymbol)) {
				parameter = null;
				return false;
			}
			
			string[] split;
			int commentIndex = text.IndexOf(CommentSymbol);
			int quote1Index = text.IndexOf(QuoteSymbol);
			
			if (commentIndex >= 0) {	// Comment in line.
				if (quote1Index >= 0) {	// Quote in line.
					if (quote1Index > commentIndex) {	// Comment comes first; quote is part of the comment.
						text = text.Substring(0, commentIndex);
					} else {							// Quote comes first; comment symbol may be part of the value;
		
						int quote2Index = (quote1Index < text.Length - 1) ?
							text.Substring(quote1Index + 1).IndexOf(QuoteSymbol) :	// Beware! Index in substring, not in whole string.
							-1;
						
						if (quote2Index < 0) {			// No 2nd quote; comment symbol cannot be quoted.
							text = text.Substring(0, commentIndex);
						} else {						// We have a 2nd quote.
							quote2Index += quote1Index + 1;
							
							if (commentIndex > quote2Index) { // TODO: Check if quote2 is escaped.
								text = text.Substring(0, commentIndex);
							} // Otherwise the comment symbol is part of the quote.
						}
					}
				}
			}
			
			
			split = text.Split(new string[] { Separator }, StringSplitOptions.RemoveEmptyEntries);
			
			if (split.Length < 2) {
				parameter = null;
				return false;
			}
			
			split[1] = split[1].Trim();
			split[1] = UnescapeNewLine(split[1]);
			parameter = new IniParameter(split[0].Trim(), split[1]);
			return true;
		}
	}
}
