﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.01.17.
 * Idő: 15:04
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of Statistics.
	/// </summary>
	public static class Statistics
	{
		private static int notNaNCount;	// The number of elements in a list that are not NaN.
 		public static bool SkipNaN = true;
 		
 		[Obsolete("Use DoubleListExtensions instead.")]
 		public static void GetExtrema(IList<double> list, out double min, out double max)
 		{
 			double x;
 			min = Double.MaxValue;
 			max = Double.MinValue;
 			
 			for (int i = 0; i < list.Count; i++) {
 				x = list[i];
 				if (x < min) min = x;
 				if (x > max) max = x;
 			}
 		}
 		
 		// Validated by comparison to LabVIEW 7.1 results.
 		[Obsolete("Use DoubleListExtensions instead.")]
 		public static double GetMean(IList<double> list)
 		{
 			if (list == null) throw new NullReferenceException("Mean cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Mean cannot be calculated:" +
 				                                                              " the list is empty.");
 			int n = 0;
 			double sum = 0;
 			
 			foreach (double element in list) {
 				// Check if we should simply skip NaN values instead of letting them NaN out the whole mean.
 				if (!SkipNaN || !Double.IsNaN(element)) {
 					sum += element;
 					n++;
 				}
 			}
 			notNaNCount = n;
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return (sum / (double) n);
 			}
 		}
 		
 		// Validated by comparison to LabVIEW 7.1 results.
 		[Obsolete("Use DoubleListExtensions instead.")]
 		public static double GetRms(IList<double> list)
 		{
 			if (list == null) throw new NullReferenceException("Root mean square cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Root mean square cannot be calculated:" +
 				                                                              " the list is empty.");
 			
 			double sum = 0;
 			int n = 0;
 			
 			foreach (double d in list) {
 				if (!SkipNaN || !Double.IsNaN(d)) {
 					sum += d * d;
 					n++;
 				}
 			}
 			
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return Math.Sqrt(sum / (double)n);
 			}
 		}
 		
 		// Validated by comparison to LabVIEW 7.1 results.
 		[Obsolete("Use DoubleListExtensions instead.")]
 		public static double GetSD(IList<double> list, bool isSample)
 		{
 			if (list == null) throw new NullReferenceException("Standard deviation cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Standard deviation cannot be calculated:" +
 				                                                              " the list is empty.");
 			
 			double distance;
 			double mean = GetMean(list);
 			double sum = 0;
 			int n = 0;
 			int w;
 			
 			foreach (double d in list) {
 				if (!SkipNaN || !Double.IsNaN(d)) {
 					distance = d - mean;
 					sum += distance * distance;
 					n++;
 				}
 			}
 			
 			notNaNCount = n;
 			w = isSample ? (n - 1) : n; 
 			if (w == 0) {
 				return Double.NaN;
 			} else {
 				return Math.Sqrt( sum / (double)w);
 			}
 		}
 		
 		// Standard error of the mean based on sample standard deviation.
 		[Obsolete("Use DoubleListExtensions instead.")]
 		public static double GetSE(IList<double> list)
 		{
 			if (list == null)
 				throw new NullReferenceException("Cannot obtain standard error: the list is a null reference.");
 			if (list.Count <= 0)
 				throw new InvalidOperationException("Cannot obtain standard error: the list is empty.");
 			double sd = Statistics.GetSD(list, true);
 			int n = SkipNaN ? notNaNCount : list.Count;
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return (sd / Math.Sqrt(n));
 			}	
 		}
		
		public static LinearFitResults GetLinearFit(IList<double> xValues, IList<double> yValues)
		{
 			if ((xValues == null) || (yValues == null))
 				throw new NullReferenceException("The fit cannot be performed: at least one of the sequences" +
			                                        " is a null reference.");
			if (xValues.Count != yValues.Count)
			     throw new InvalidOperationException("The fit cannot be performed: the numbers of" +
			                                        " points in the sequences differ.");
			double sum0 = 0.0;
			double sum1 = 0.0;
			double sum2 = 0.0;
			double sum3 = 0.0;
			double sum4 = 0.0;
			double meanX;
			double meanY;
			double x;
			double y;
			int n = xValues.Count;
			LinearFitResults results = new LinearFitResults();
			
			for (int i = 0; i < n; i++) {
				sum0 += xValues[i];
				sum1 += yValues[i];
			}
			
			meanX = sum0 / (double)n;
			meanY = sum1 / (double)n;
			sum0 = 0.0;
			sum1 = 0.0;
			
			for (int i = 0; i < n; i++) {
				x = xValues[i];
				y = yValues[i];
				sum0 += x * y - meanX * meanY;
				sum1 += x * x - meanX * meanX;
				sum2 += (x - meanX) * (y - meanY);
				sum3 += (x - meanX) * (x - meanX);
				sum4 += (y - meanY) * (y - meanY);
			}
			
			results.Slope = sum0 / sum1;
			results.RSquare = (sum2 * sum2) / (sum3 * sum4);
			results.Intercept = meanY - results.Slope * meanX;

			return results;
		}
	
	}
}
