﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2009.09.22.
 * Idő: 10:18
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.SignalAnalysis
{
	public enum WidthUnit {
		Absolute,
		Index
	}
	
	/// <summary>
	/// Description of MovingAverage.
	/// </summary>
	public class MovingAverage : ISignalTransformation
	{
		protected double width;
		protected WidthUnit widthUnit;
		
		
		public MovingAverage(double width, WidthUnit widthUnit)
		{
			this.width = width;
			this.widthUnit = widthUnit;
		}
		
		public MovingAverage(double width) : this(width, WidthUnit.Absolute) { }
		
		public MovingAverage() : this(1.0, WidthUnit.Absolute) { }
		
		
		
		public double Width {
			get { return width; }
			set { width = value; }
		}
		
		public WidthUnit WidthUnit {
			get { return widthUnit; }
			set { widthUnit = value; }
		}
		
		
		
		public ISignal Transform(ISignal original)
		{
			ISignal filtered = null;
			double increment = Double.NaN;
			if (original is ScaledSignal) {
				filtered = new ScaledSignal((original as ScaledSignal).XScale.Clone());
				increment = (original as ScaledSignal).XScale.Increment;
			} else if (original is UnevenSignal) {
				filtered = new UnevenSignal(original.Count);
				increment = original.X[1] - original.X[0];
			}
			int widthInIndices = (widthUnit == WidthUnit.Absolute) ?
				(int)Math.Round(width / increment, MidpointRounding.AwayFromZero) :
				(int)width;
			
			for (int i = 0; i < filtered.Count; i++) {
				filtered[i] = GetLocalAverage(original, i, widthInIndices);
			}
			
			return filtered;
		}
		
		protected double GetLocalAverage(IList<double> list, int index, int widthInIndices)
		{
			int minIndex = index - widthInIndices;
			int maxIndex = index + widthInIndices;
			int count = 0;
			double sum = 0.0;
			
			if (minIndex < 0) minIndex = 0;
			if (maxIndex >= list.Count) maxIndex = list.Count - 1;
			
			for (int i = minIndex; i <= maxIndex; i++) {
				sum += list[i];
				count++;
			}
			
			return sum / count;
		}
	}
}
