﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.09.17.
 * Time: 13:48
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.SignalAnalysis
{
	/// <summary>
	/// Description of PeakDetector.
	/// </summary>
	public class PeakDetector : ISignalTransformation
	{
		protected double filterTime = 20.0;
		protected int sensitivity = 50;
		protected double preInterval = 0.02;
		protected double postInterval = 0.05;
		
		public ISignal Transform(ISignal original)
		{
			double dt = (original is ScaledSignal) ? (original as ScaledSignal).XScale.Increment :
				original.X[1] - original.X[0];
			List<int> peakIndices = FindPeaks(original.Y, dt, -1.0);
			UnevenSignal peaks = new UnevenSignal(peakIndices.Count);
			
			for (int i = 0; i < peakIndices.Count; i++) {
				peaks.Add(original.X[peakIndices[i]], original[peakIndices[i]]);
			}
			
			return peaks;
		}
		
		// Method created by Zoltán Gingl.
		public List<int> FindPeaks(IList<double> original, double dt, double maxT)
		{
			double segmentT = 2.0;
			int segmentWidth = (int)(segmentT / dt);	// 2 seconds for adaptive threshold.
			int n = (maxT < 0.0) ? original.Count : (int)((maxT + segmentT / 2)/ dt + 1);
			double[] originalY = new double[original.Count];
			List<int> peakIndices = new List<int>(n);
			int filterCount;
			int j;
			int l;
			int k;
			int peakCount = 0;
			double threshold;
			double threshold1;
			double threshold2;
			double ydp;
			double mean = 0.0;

			original.CopyTo(originalY, 0);

			// Moving average, 20 ms:
			filterCount = (int)(filterTime / 1000.0 /dt + 0.5);
			if (filterCount > 0 && filterCount < 100) {
				for (int i = 0; i < n - filterCount; i++) {
					mean = 0.0;
					for (j = i; j < i + filterCount; j++) mean += original[j];
					originalY[i] = mean /= filterCount;
				}
			}
			// Derivative: 
			for (int i = 0; i < n - 1; i++) {
				originalY[i] = originalY[i + 1] - originalY[i];
			}
			
			originalY[n - 1] = originalY[n - 2];
	
			k = (int)(0.1 / dt);  			// Update adaptive threshold time.
			filterCount = n / segmentWidth; // Segments.
			l = 0;
			ydp = 0.0;
			threshold1 = 0.0;
			for (j = 0; j < segmentWidth; j++) if (threshold1 > originalY[j]) threshold1 = originalY[j]; // Initialise threshold.
			threshold1 = threshold1 * (100 - sensitivity) / 100.0;
			threshold2 = 0.0;
			for (j = 0; j < segmentWidth; j++) threshold2 += originalY[j] * originalY[j];
			threshold2 = -3.0 * Math.Sqrt(threshold2 / segmentWidth);
			threshold = threshold1;	// (thres1 + thres2) / 2;
			for (int i = 0; i < n - segmentWidth / 2; i++) {
				if (ydp >= threshold && originalY[i] < threshold && (i - l) > (0.1 / dt)) {
					peakIndices.Add(i);
					peakCount++;
					l = i;
				}
				if (i % k == k - 1) { // Recalculate threshold:
					j = i - segmentWidth / 2;
					if (j <0 ) j = 0;
					for (threshold1 = 0.0; j < i + segmentWidth / 2; j++) if (threshold1 > originalY[j]) threshold1 = originalY[j];	// Initialise threshold.
					threshold1 = threshold1 * (100 - sensitivity) / 100.0;
					for (threshold2 = 0.0; j < segmentWidth; j++) threshold2 += originalY[j] * originalY[j];
					if (i >= segmentWidth / 2) {
						threshold2 = -3.0 * Math.Sqrt(threshold2 / segmentWidth);
					} else {
						threshold2 = -3.0 * Math.Sqrt(threshold2/ (segmentWidth /2 + i));
					}
					threshold = threshold1; // (thres1 + thres2) / 2;
				}
			}
			
			// Adjust peak detection markers now:
			for(j = 0; j < peakCount; j++) {
				int imax;
				double ymax;
				int i = peakIndices[j] - (int)(preInterval / dt);
				if (i < 0) i = 0;
				imax = peakIndices[j] + (int)(postInterval / dt);
				if (imax > n) imax = n;
				k = i;
				ymax = original[i];
				for( ; i < imax; i++) {
					if (ymax < original[i]) {
						ymax = original[i];
						k = i;	// Update position.
					}
				}
				peakIndices[j] = k;
			}
			
			
			return peakIndices;
		}
		
		public List<int> FindPeaks(IList<double> original, double dt)
		{
			return FindPeaks(original, dt, -1.0);
		}
	}
}
